/*
 * JBoss, Home of Professional Open Source
 * Copyright 2006, JBoss Inc., and others contributors as indicated 
 * by the @authors tag. All rights reserved. 
 * See the copyright.txt in the distribution for a
 * full listing of individual contributors. 
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU Lesser General Public License, v. 2.1.
 * This program is distributed in the hope that it will be useful, but WITHOUT A 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License,
 * v.2.1 along with this distribution; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * (C) 2005-2006,
 * @author Mark Little (mark.little@jboss.com)
 */

import com.arjuna.ats.jta.xa.XidImple;
import com.arjuna.ats.arjuna.common.Uid;

import javax.transaction.xa.XAException;
import javax.transaction.xa.XAResource;
import javax.transaction.xa.Xid;
import org.apache.log4j.Category;
import java.io.*;
import javax.naming.*;

public class ExampleXAResource2
   implements XAResource, Referenceable, Serializable
{
   private final static long _sleepTime = 20000;
   private Xid _currentXid;
   private Reference _reference;

   public ExampleXAResource2()
   {
      myLog("ExampleXAResource2 (Constructor)");
   }

   /**
    *
    * @param param1 <description>
    * @param param2 <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public void start(Xid xid, int flags) throws XAException
   {
      myLog("start");
      _currentXid = xid;
   }

   /**
    *
    * @param param1 <description>
    * @param param2 <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public void end(Xid xid, int flags) throws XAException
   {
      myLog("end");
      _currentXid = null;
   }

  /**
    *
    * @param param1 <description>
    * @param param2 <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public void commit(Xid xid, boolean onePhase) throws XAException
   {
      myLog("commit,xid=" + xid + ",onePhase=" + onePhase);
      
      mySleep(_sleepTime);
   }

   /**
    *
    * @param param1 <description>
    * @return <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public int prepare(Xid xid) throws XAException
   {
      myLog("prepare "+xid);

      int i = 2;
      if (i == 1)
      {
        throw(new XAException(XAException.XA_RBROLLBACK));
      }

      return XA_OK;
   }

   /**
    *
    * @param param1 <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public void rollback(Xid xid) throws XAException
   {
      myLog("rollback");
   }

   /**
    *
    * @param param1 <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public void forget(Xid xid) throws XAException
   {
      myLog("forget");
   }

   /**
    *
    * @param param1 <description>
    * @return <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public Xid[] recover(int flag) throws XAException
   {
      myLog("recover");

      Xid[] xids = null;

      if (xid != null)
      {
	  xids = new Xid[1];
      
	  xids[0] = new XidImple(new Uid(xid));

	  System.err.println("expect recovery on "+xids[0]);
      }
      
      return xids;
   }

   /**
    *
    * @param param1 <description>
    * @return <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public boolean isSameRM(XAResource other) throws XAException
   {
      myLog("isSameRM");
      return(false);
   }

   /**
    *
    * @return <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public int getTransactionTimeout() throws XAException
   {
      myLog("getTransactionTimeout");
      return 10;
   }

   /**
    *
    * @param param1 <description>
    * @return <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public boolean setTransactionTimeout(int seconds) throws XAException
   {
      myLog("setTransactionTimeout");
      return true;
   }

  private void myLog (String msg)
  {
    try
    {
      String fileName = "ExampleXAResource2.log";
      FileWriter fw = new FileWriter(fileName, true);
      PrintWriter pw = new PrintWriter(fw);
      pw.println(msg);
      pw.flush();
      pw.close();
      fw.close();
        System.out.println("ExampleXAResource2: "+msg);
    }
    catch (Throwable ex)
    {
      System.out.println("Caught an exception");
      ex.printStackTrace();
    }

  }

  private void mySleep(long millis)
  {
    myLog("Sleeping " + millis + " milliseconds");
    try
    {
      Thread.sleep(millis);
    }
    catch (Exception ex)
    {
    }
    myLog("Sleep complete");
  }

   public void setReference(Reference _reference)
   {
      myLog("setReference, _reference="+_reference);
      this._reference = _reference;
   }

   public Reference getReference() throws NamingException
   {
      myLog("getReference");
      return _reference;
   }

   private void writeObject(java.io.ObjectOutputStream out) throws IOException
   {
      myLog("writeObject (Serialized)");
   }

   private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException
   {
      myLog("readObject (Deserialized");
   }

    public static String xid;
    
}
